module MyMenuHelper
  include RailsConnector::MenuHelper

  def top_menu
    #TODO "Wegweiser" and "Kundeservice" have been blocked
    menu(Obj.find_by_path('/internet/playland/de'), @obj, :max_depth => 1, :menu_id => 'nav_main') do |items, level|
      items.reject{|obj| obj.obj_class == "Stylesheet"}.sort_by {|obj| obj[:ip_sortOrder] || ''}.reject { |e| %w( wegweiser service ).include?(e.permalink)  }
    end
  end

  def side_menu
    return unless @obj.ancestors.length > 3
    menu(@obj.ancestors.length > 4 ? @obj.ancestors[4] : @obj, @obj,
        :max_depth => 2,
        :menu_id => 'submenu') do |items, level|
      items.reject{|obj| obj.obj_class == "Stylesheet"}.sort_by {|obj| obj[:ip_sortOrder] || ''}
    end
  end

  # Renders a menu which consists of simple HTML markup, i.e. nested <tt><ul></tt> lists within a
  # <tt><div></tt> tag.
  # <em>start_obj</em> ist the topmost Obj that serves as the root of the menu. This
  # root object itself is not shown in the menu.
  # <em>current_obj</em> is an optional Obj which is used to mark the currently active item.
  # You may specify the following <em>options</em>:
  #
  # [:menu_id] the ID of the surrounding <tt><div></tt> tag.
  # [:max_depth] the maximum number of levels the object tree is traversed. The default is <tt>:unlimited</tt>.
  # [:unfold => :all] expands all branches.
  # [:current_time] the time for which the toclist is evaluated, use PreviewTime.preview_time when using the TimeMachineHelper.time_machine.
  # [:hide_forbidden] hide entries for objects the user is not permitted to access, default is false.
  #
  # The menu hierarchy starts with the active children of <tt>start_obj</tt>.
  #
  # Only those branches are rendered that are on the path to the <tt>current_obj</tt>.
  # In order to open all branches, e.g. to build a sitemap, specify <tt>:unfold => :all</tt>.
  #
  # The depth of the menu can be limited by <tt>:max_depth</tt>. It defaults to <tt>:unlimited</tt>.
  # Rendering stops at <tt>current_obj</tt>'s children or at <tt>:max_depth</tt>, whichever is
  # reached first.
  #
  # If an item gets a CSS class the class is given to the <tt><li></tt> and to the enclosed
  # <tt><a></tt> or <tt><span></tt> tag.
  #
  # The CSS class 'first' is given to the first item of a children list.
  #
  # The CSS class 'last' is given to the last item of a children list.
  #
  # All menu items except the one for <tt>current_obj</tt> have a link to their
  # respective object. The menu item for the <tt>current_obj</tt> contains a
  # <tt><span></tt> tag. This item is given the CSS class 'active'.
  #
  # If the <tt>current_obj</tt> is not displayed by the menu (because of <tt>:max_depth</tt>
  # restrictions) the deepest displayed item on the path to <tt>current_obj</tt> is given the CSS
  # class 'active'.
  #
  # If an item is opened (because it is on the path to <tt>current_obj</tt> or <tt>:unfold =>
  # :all</tt> was specified) it is given the CSS class 'open'.
  #
  # If an item has children and is not opened it is given the CSS class 'closed'.
  #
  # The CSS classes 'open' and 'closed' are NOT given to items on the last level displayed.
  #
  # Specify <tt>:menu_id</tt> to give the surrounding <tt><div></tt> a CSS ID.
  #
  # For example, assume the following hierarchy:
  # * Root
  #   * Child 1
  #     * Subchild 1
  #     * Subchild 2
  #   * Child 3
  #   * Child 2
  #
  # If you type
  #
  #   menu(Obj.root, @obj, :max_depth => 1)
  #
  # the helper will generate, if <tt>@obj</tt> is "Subchild 1":
  #
  #   <div>
  #     <ul class="level-1">
  #       <li class="first"><a href="/2011/child1" class="first">Child 1</a></li>
  #       <li><a href="/2013/child3">Child 3</a></li>
  #       <li class="last"><a href="/2012/child2" class="last">Child 2</a></li>
  #     </ul>
  #   </div>
  #
  # If you type
  #
  #   menu(@obj.parent, @obj, :max_depth => 2, :menu_id => 'submenu')
  #
  # the helper will generate, if <tt>@obj</tt> is "Child 1":
  #
  #   <div id="submenu">
  #     <ul class="level-1">
  #       <li class="active first"><span class="active first">Child 1</span>
  #         <ul class="level-2">
  #           <li><a href="/2021/subchild1">Subchild 1</a></li>
  #           <li><a href="/2022/subchild2">Subchild 2</a></li>
  #         </ul>
  #       </li>
  #       <li><a href="/2013/child3">Child 3</a></li>
  #       <li class="last"><a href="/2012/child2" class="last">Child 2</a></li>
  #     </ul>
  #   </div>
  #
  # The menu items of each level can be sorted, filtered etc. by providing a
  # block which takes the objects and the level number for a menu level and
  # returns a new list.
  #
  # If you type
  #
  #   menu(Obj.root, nil, :max_depth => :unlimited, :unfold => :all) do |items, level|
  #     items.reject{|obj| obj.obj_class == "ip_stylesheet"}.sort_by {|obj| obj.sort_value || ''}
  #   end
  #
  # and the obj_class of "Subchild 1" is "ip_stylesheet", the helper will generate:
  #
  #   <div>
  #     <ul class="level-1">
  #       <li class="active first"><span class="active first">Child 1</span>
  #         <ul class="level-2">
  #           <li><a href="/2022/subchild2">Subchild 2</a></li>
  #         </ul>
  #       </li>
  #       <li><a href="/2012/child2">Child 2</a></li>
  #       <li class="last"><a href="/2013/child3" class="last">Child 3</a></li>
  #     </ul>
  #   </div>
  #
  # As you can see, there is no "Subchild 1" and the order of "Child 2" and "Child 3" is as
  # expected.
  #
  # === NOTE
  #
  # Build your own helpers around this one to keep the logic out of your
  # views. Put them in <tt>app/helpers/my_menu_helpers.rb</tt> and add the line
  # <tt>helper MyMenuHelper</tt> in your controller. For examples, take a look at
  # the helper generated by the plugin.
  #
  def menu(start_obj, current_obj, options = {}, &block) # :yield: items, level
    menu_items = build_menu_tree(children(start_obj, options), current_obj, options, &block)
    content_tag(:div, render_menu(1, menu_items), :id => options[:menu_id])
  end

  private

  class MenuItem #:nodoc: all
    attr_accessor :open, :closed, :active, :current, :first, :last, :children, :obj
  end

  ## BUILDER METHODS

  def build_menu_tree(objects, current_obj, options, &block)
    options.reverse_merge! :max_depth => :unlimited, :hide_forbidden => false

    build_menu_tree_for_level(1, objects, current_obj, options, &block)
  end

  def build_menu_tree_for_level(level, objects, current_obj, options, &block)
    menu_items = []

    objects = block.call(objects, level) if block_given?
    objects.reject!{|obj| !is_permitted obj } if options[:hide_forbidden]
    objects.each_with_index do |child, i|
      menu_item = MenuItem.new

      menu_item.open = opened?(level, child, current_obj, options)
      menu_item.closed = closed?(level, child, current_obj, options)
      menu_item.active = active?(level, child, current_obj, options)
      menu_item.first = i == 0
      menu_item.last = i == objects.size - 1
      menu_item.current = child == current_obj

      menu_item.obj = child

      toclist = compute_children(options, level, current_obj, child)
      menu_item.children = build_menu_tree_for_level(level + 1, toclist, current_obj, options, &block)

      menu_items << menu_item
    end

    menu_items
  end

  def opened?(level, child, current_obj, options)
    !last_level?(level, options) && (
          options[:unfold] == :all ||
          current_obj == child ||
          (current_obj && current_obj.ancestors.include?(child))
        )
  end

  def closed?(level, child, current_obj, options)
    !last_level?(level, options) &&
        !opened?(level, child, current_obj, options)
  end

  def compute_children(options, level, current_obj, child)
    if opened?(level, child, current_obj, options)
      children(child, options)
    else
      []
    end
  end

  def last_level?(level, options)
    options[:max_depth] != :unlimited && level == options[:max_depth]
  end

  def children(child, options)
    child.toclist(options[:current_time])
  end

  def active?(level, child, current_obj, options)
    current_obj &&
        (child == current_obj ||
          (last_level?(level, options) && current_obj.ancestors.include?(child)))
  end

  ## RENDERING METHODS

  def render_menu(level, menu_items)
    return "" if menu_items.empty?

    li_tags = menu_items.map do |menu_item|
      entry = render_menu_item(menu_item)
      entry << render_menu(level + 1, menu_item.children)
      content_tag(
        :li, entry, {:class => css_classes_for_menu_item(menu_item)}
      )
    end

    content_tag(:ul, li_tags, :class => "level-#{level}") unless li_tags.blank?
  end

  def render_menu_item(menu_item)
    tag_options = {:class => css_classes_for_menu_item(menu_item)}
    label = render_menu_item_label(menu_item)
    menu_item.current ?
      content_tag(:span, label, tag_options) :
      link_to(label, cms_path(menu_item.obj), tag_options)
  end

  def render_menu_item_label(menu_item)
    display_value(menu_item.obj.display_title)
  end

  def css_classes_for_menu_item(menu_item)
    classes = %w{open closed first last active}.select { |klass| menu_item.send(klass) }.join(" ")

    # return nil to avoid generating empty class attributes
    # we dont want <span class="">, we want <span>
    classes.blank? ? nil : classes
  end
end
