set :application, "playland"
set :repository, "svn://svn/nps/trunk/Apps/#{application}"

set :deploy_via, :copy
set :copy_strategy, :export
set :user, 'develop'
set :deploy_to, "/home/#{user}/#{application}"
set :use_sudo, false

# Use the command line switch -S to preselect a stage.
# Example:
#   cap -S stage=production deploy
unless fetch(:stage, nil)
  set :stage do
    Capistrano::CLI.ui.choose do |menu|
      menu.header = "The following stages are available"
      menu.choice "preview"
      menu.choice "staging"
      menu.choice "production"
      menu.prompt = "Please choose: "
    end
  end
end

set :rails_env, stage

case rails_env
when "preview"
  role :web, "vivaldi1"
  role :app, "vivaldi1"
  role :db,  "vivaldi1", :primary => true
  set :prefix, '/default'
when "staging"
  role :web, "vivaldi1"
  role :app, "vivaldi1"
  role :db,  "vivaldi1", :primary => true
when "production"
  role :web, "vivaldi8"
  role :app, "vivaldi8"
  role :db,  "vivaldi8", :primary => true
end

namespace :deploy do
  %w(start stop restart).each do |action|
    desc "#{action.capitalize} the application servers."
    task action, :roles => :app do
      config = {
        "address" => "127.0.0.1",
        "port" => 20000,
        "servers" => 2,
        "pid_file" => "tmp/pids/mongrel.pid",
        "environment" => rails_env,
      }
      config["prefix"] = prefix if fetch(:prefix, nil)
      put config.to_yaml, "#{current_path}/config/mongrel_cluster.yml"
      run "sh -c 'cd #{current_path} && mongrel_rails cluster::#{action}'"
    end
  end
end


before "deploy:setup", :db
after "deploy:update_code", "db:symlink"
after "deploy:update_code", "db:mysql:infopark_blob_streaming"

namespace :db do
  desc "Create database.yml in shared path"
  task :default do
    run "mkdir -p #{shared_path}/config"
    put File.read("config/database.yml.template"), "#{shared_path}/config/database.yml"
  end

  desc "Make symlink for database.yml"
  task :symlink do
    run "ln -nfs #{shared_path}/config/database.yml #{release_path}/config/database.yml"
  end

  namespace :mysql do
    desc "Compiles the MySQL blob extension"
    task :infopark_blob_streaming do
      run "sh -c 'cd #{release_path} && rake db:mysql:infopark_blob_streaming'"
    end
  end
end


depend :remote, :command, "rake"
depend :remote, :gem, "mysql", ">=2.7"
depend :remote, :gem, "rails", "=1.2.5"
depend :remote, :gem, "json", ">=1.1.1"
depend :remote, :gem, "mime-types", ">=1.15"
depend :remote, :gem, "maruku", ">=0.5.7"
depend :remote, :gem, "mongrel", ">=1.0.1"
depend :remote, :gem, "mongrel_cluster", ">=1.0.2"
