module ObjExtensions

  # Infopark Rails Connector enables the developer to define specific behavior
  # for Fiona CMS classes by using Single Table Inheritance
  # (http://www.martinfowler.com/eaaCatalog/singleTableInheritance.html).
  #
  # If you wish to decorate the model Obj with extra behavior, you will need to
  # add it here.
  #
  # In addition, you will need to edit your environment(s) accordingly:
  #
  #     def init_rails_connector
  #       ObjExtensions.enable
  #       ...
  #     end

  def self.enable
    Obj.class_eval do

      # This method belongs to the Obj that has a voting associated with.
      def has_voting?
        !(self[:ip_votingSelected] || []).empty?
      end

      # This method belongs to the Obj that has a voting associated with.
      def voting
        v = (self[:ip_votingSelected] || []).first
        v.destination_object if v
      end

      # Returns a list of exportable? binary children.
      def toclist_binary(current_time = nil)
        children.find(:all).select do |child|
          child.binary? && child.exportable?(current_time)
        end
      end

      def allow_rating?
        self[:ip_allowRating] == "yes"
      end

      def allow_comments?
        self[:ip_allowComments] == "yes"
      end

      def seo_description
        descr = (self[:ip_description] unless self[:ip_description].blank?) || body
        HTML::FullSanitizer.new.sanitize(descr.to_s.strip.gsub(%r{[\n|\r]}, " ")).mb_chars[0,300]
      end

      def included_in_seo_sitemap?
        return true if homepage?
        super && ancestors.include?(NamedLink.get_object("homepage_de"))
      end

      def self.homepage
        NamedLink.get_object("homepage_de", :cache => false)
      end

      # Gewisse Objekte benötigen eigene Vorlagen. Um das ganze etwas zu
      # vereinfachen, soll das Objekt selber sagen, welches Template es zu
      # verwenden wünscht und das Ergebnis soll dann an render übergeben
      # werden.
      def template
        if self == NamedLink.get_object('newsletter_de')
          return 'newsletter'
        end
        if self == NamedLink.get_object('podcast_de')
          return 'podcasts'
        end
        if ['Newsletter','Event','JobOffer','Faq','Shop'].include?(self.obj_class)
          self.obj_class.gsub(/(ip_.*)/,'\1').underscore
        else
          return 'index'
        end
      end
    end
  end
end
