require File.dirname(__FILE__) + '/../spec_helper'

describe CmsController, "#route_for" do

  it "should map { :controller => 'cms', :action => 'index', :id => '1' } to /1" do
    route_for(:controller => "cms", :action => "index", :id => "1").should == "/1"
  end

  it "should map { :controller => 'cms', :action => 'index', :id => '1', :name => 'a' } to /1/a" do
    route_for(:controller => "cms", :action => "index", :id => "1", :name => 'a').should == "/1/a"
  end

  it "should map { :controller => 'cms', :action => 'index', :permalink => ['a', 'b'] } to /a/b" do
    route_for(:controller => "cms", :action => "index", :permalink => ['a', 'b']).should == "/a/b"
  end

  it "should map { :controller => 'cms', :action => 'index', :permalink => 'a/b' } to /a/b" do
    route_for(:controller => "cms", :action => "index", :permalink => 'a/b'.split("/")).should == "/a/b"
  end

  it "should check wether '' takes home route" do
    opts = {:controller => "cms", :action => "index"}
    assert_routing "", opts
  end

  it "should check permalink route" do
    opts = {:controller => "cms", :action => "index", :permalink => ['my_permalink'] }
    assert_routing "/my_permalink", opts
  end

end

describe CmsController, "if requesting homepage" do
  before do
    controller.stub!(:ensure_object_is_permitted).and_return(true)
    controller.stub!(:ensure_object_is_active).and_return(true)
    controller.stub!(:set_google_expire_header)
    controller.stub!(:index)
    @obj = mock_model(Obj, :mime_type => 'text/html', :reload => true)
  end

  it "should load the homepage object defined by named link if neither an id nor permalink param is given" do
    NamedLink.should_receive(:get_object).with("homepage_de", :cache => false).and_return(@obj)
    get 'index'
  end

  it "should load the object with the specified id, if given" do
    NamedLink.should_not_receive(:get_object).with("homepage_de")
    Obj.stub!(:find).with("123")
    get 'index', :id => 123
  end

  it "should load the object with the specified permalink, if given" do
    NamedLink.should_not_receive(:get_object).with("homepage_de")
    Obj.stub!(:find_by_permalink!).with("some_page")
    get 'index', :permalink => "some_page"
  end

end

describe CmsController, "before filter forward_wiki" do
  before do
    controller.stub!(:ensure_object_is_permitted).and_return(true)
    controller.stub!(:ensure_object_is_active).and_return(true)
    controller.stub!(:set_google_expire_header)
    controller.stub!(:load_object)
  end

  def do_request
    get 'index'
  end

  it "should redirect wiki objects to the wiki controller" do
    @obj = mock_model(Wiki, :mime_type => 'text/html', :to_param => 123)
    controller.instance_variable_set("@obj", @obj)
    do_request
    response.should redirect_to(:controller => :wiki, :id => '123')
  end

  it "should not redirect all other objects" do
    @obj = mock_model(Obj, :mime_type => 'text/html', :id => 123, :template => 'index')
    controller.instance_variable_set("@obj", @obj)
    do_request
    response.should_not be_redirect
  end
end

describe CmsController, "before filter ensure_object_is_permitted, where the user is not permitted to see the object" do
  before do
    controller.stub!(:load_object).and_return(true)
    @obj = mock_model(Obj, :mime_type => 'text/html', :hide_contents! => nil, :permitted_groups => ["registered"])
    controller.instance_variable_set("@obj", @obj)
    controller.stub!(:ensure_object_is_active).and_return(true)
  end

  it "should stop the filter chain" do
    controller.should_not_receive(:index)
    get 'index'
  end

  it "should render an error view" do
    get 'index'
    response.should render_template("errors/403_forbidden")
  end

  it "should make the object hide its contents" do
    @obj.should_receive(:hide_contents!)
    get 'index'
  end
end