require File.dirname(__FILE__) + '/../spec_helper'

def find_rateable_obj
  Obj.stub!(:find).and_return(@obj = mock_model(
    Obj, :allow_rating? => true, :allow_anonymous_rating? => true
  ))
end

describe RatingsController, "AJAX /reset for a page where the user is admin" do
  before do
    find_rateable_obj
    controller.stub!(:admin?).and_return(true)
    @obj.stub!(:reset_rating)
  end

  def do_ajax
    xhr :post, :reset, :obj_id => "2001"
  end

  it "should fetch the object" do
    Obj.should_receive(:find).with("2001").and_return(@obj)
    do_ajax
    assigns[:obj].should == @obj
  end

  it "should reset the rating" do
    @obj.should_receive(:reset_rating)
    do_ajax
  end

  it "should redisplay the rating" do
    do_ajax
    response.should render_template("reset")
  end
end

describe RatingsController, "AJAX /reset for a page where the user is an ordinary user" do
  before do
    find_rateable_obj
    controller.stub!(:admin?).and_return(false)
  end

  def do_ajax
    xhr :post, :reset, :obj_id => "2001"
  end

  it "should NOT reset the rating" do
    @obj.should_not_receive(:reset_rating)
    do_ajax
  end
end
