require File.dirname(__FILE__) + '/../spec_helper'

describe VotesController, "#route_for" do
  it "should map { :controller => 'votes', :action => 'vote', :obj_id => '15' } to /votes/vote/15" do
    route_for(:controller => "votes", :action => "vote", :obj_id => "15").should == "/votes/vote/15"
  end
end

describe VotesController, "AJAX /vote/2001 with choice 'Teddy'" do
  include VotesHelper

  before do
    Obj.stub!(:find).and_return(@voting = mock_model(Obj))
    @voting.stub!(:vote).and_return(true)
  end

  def do_ajax
    xhr :post, :vote, :obj_id => '2001', :choice => 'Teddy'
  end

  it "should fetch the voting object with ID 2001" do
    Obj.should_receive(:find).with("2001").and_return(@voting)
    do_ajax
  end

  it "should vote for the given answer" do
    @voting.should_receive(:vote).with('Teddy')
    do_ajax
  end

  it "should display the voting result" do
    do_ajax
    response.should render_template("_result")
  end

  it "should remember that the user has voted in order to prevent the user from voting again" do
    lambda { do_ajax }.should change { current_user_has_already_voted?(@voting) }.to(true)
  end
end


describe VotesController, "GET /index when current user is admin" do
  before do
    controller.stub!(:admin?).and_return(true)
  end

  def do_get
    get :index
  end

  it "should fetch all votings" do
    Voting.should_receive(:find).with(:all)
    do_get
  end
end


describe VotesController, "GET /index when current user is an ordinary user" do
  before do
    controller.stub!(:admin?).and_return(false)
  end

  def do_get
    get :index
  end

  it "shouldn't fetch votings" do
    Obj.should_not_receive(:find_all_by_obj_class).with("Voting")
    do_get
  end

  it "should redirect to the homepage" do
    do_get
    response.should be_redirect
  end
end
