require File.dirname(__FILE__) + '/../spec_helper'

describe MyMenuHelper do

  describe "(top_menu)" do
    it "should render a list of navigation links" do
      helper.top_menu.should have_tag('ul.level-1') do
        with_tag('li.first a[href=?]', '/wir-ueber-uns')
        with_tag('li a[href=?]', '/marken')
        with_tag('li a[href=?]', '/presse')
        with_tag('li.last a[href=?]', '/investor-relations')
      end
    end
  end

  describe "(side_menu)" do
    before do
      assigns[:obj] = (@obj = Obj.find_by_name('wirueberuns'))
      @obj.toclist.each do |o|
        helper.stub!(:cms_path).with(o).and_return("/cms/#{o.name}")
      end
    end

    it "should render a list of navigation links" do
      helper.side_menu.should have_tag('ul.level-1') do
        with_tag('li.closed a[href=?]', '/cms/verantwortung')
        with_tag('li.closed a[href=?]', '/cms/faq')
        with_tag('li.closed a[href=?]', '/cms/geschichte')
        with_tag('li.closed a[href=?]', '/cms/stellenanzeigen')
        with_tag('li.closed a[href=?]', '/cms/impressum')
        with_tag('li.closed a[href=?]', '/cms/geschichte2')
      end
    end
  end

  describe '(menu)' do
    describe "menu, one level hierarchy" do

      before do
        @children = []
        @children << mock_model(Obj, :display_title => "child1's title", :toclist => [])
        @children << @current = mock_model(Obj, :display_title => 'child2"s title', :toclist => [])
        @children << mock_model(Obj, :display_title => "child3's title", :toclist => [])
        @obj = mock_model(Obj, :toclist => @children)
        helper.stub!(:cms_path).and_return("/path")
        @current.stub!(:ancestors).and_return [@obj]
      end

      def do_menu
        helper.menu(@obj, @current)
      end

      it "should render a one-level deep menu starting at the given object" do
        do_menu.should have_tag('ul', 1)
        do_menu.should have_tag('ul > li', 3)
      end

      it "should link all items but current" do
        do_menu.should have_tag('ul li a[href=/path]', 2)
        do_menu.should have_tag('ul') do
          with_tag('li:first-child a[href=/path]')
          with_tag('li:last-child a[href=/path]')
        end
      end

      it "should wrap the current object's item in a <span>" do
        do_menu.should have_tag('ul li:nth-child(2) span')
      end

      it "should render the quoted titles" do
        do_menu.should have_tag('ul') do
          with_tag('li', "child1's title")
          with_tag('li', "child2&quot;s title")
          with_tag('li', "child3's title")
        end
      end

      it "should mark the list with the CSS class 'level-1'" do
        do_menu.should have_tag('ul.level-1')
      end

      it "should mark the first item with CSS class 'first'" do
        do_menu.should have_tag('li:first-child.first>a.first')
      end

      it "should mark the last item with CSS class 'last'" do
        do_menu.should have_tag('li:last-child.last>a.last')
      end

      it "should mark the current item with CSS class 'active'" do
        do_menu.should have_tag('li:nth-child(2).active>span.active')
      end

      it "should render a surrounding 'div'" do
        do_menu.should have_tag('div')
      end
    end


    describe "called without current_obj" do

      before do
        @children = []
        @children << mock_model(Obj, :display_title => "child1's title", :toclist => [])
        @children << mock_model(Obj, :display_title => "child2's title", :toclist => [])
        @children << mock_model(Obj, :display_title => "child3's title", :toclist => [])
        @obj = mock_model(Obj, :toclist => @children)
        helper.stub!(:cms_path).and_return("/path")
      end

      def do_menu
        helper.menu(@obj, nil)
      end

      it "should link all items" do
        do_menu.should have_tag('ul li a[href=/path]', 3)
      end
    end


    describe "with option :menu_id given" do

      before do
        @obj = mock_model(Obj,
        :toclist => [mock_model(Obj, :display_title => "child1's title", :toclist => [])])
        helper.stub!(:cms_path).and_return("/path")
      end

      it "should use this css ID for <div>" do
        helper.menu(@obj, nil, :menu_id => 'my_nav').should have_tag('div#my_nav', 1)
      end
    end


    describe "with option :hide_forbidden" do

      before do
        @obj = mock_model(Obj, :toclist => [
            @unpermitted = mock_model(Obj, :display_title => "unperm", :toclist => []),
            @permitted = mock_model(Obj, :display_title => "perm", :toclist => [
                @unpermitted_child = mock_model(Obj, :display_title => "unperm child", :toclist => [])
            ])
        ])
        helper.stub!(:cms_path).and_return("/path")
        #stub!(:is_permitted).and_return(true)
      end

      it "should not show children with live permissions" do
        helper.should_receive(:is_permitted).with(@permitted).and_return(true)
        helper.should_receive(:is_permitted).with(@unpermitted).and_return(false)
        helper.should_receive(:is_permitted).with(@unpermitted_child).and_return(false)
        helper.menu(@obj, @permitted, :hide_forbidden => true).should have_tag('li', 1)
      end
    end


    describe "without <li> to render" do

      before do
        @obj = mock_model(Obj, :toclist => [])
      end

      it "should render no <ul> at all" do
        helper.menu(@obj, nil).should_not have_tag('ul')
      end
    end


    describe "with two levels" do

      before do
        @children = []
        @children << mock_model(Obj, :display_title => "child1's title",
        :toclist => [mock_model(Obj, :display_title => "child11's title", :toclist => [])])
        @children << mock_model(Obj, :display_title => "child2's title", :toclist => [])
        @children << pub = mock_model(Obj, :display_title => "child3's title",
          :toclist => [
            mock_model(Obj, :display_title => "child31's title", :toclist => []),
            @current = mock_model(Obj, :display_title => "child32's title", :toclist => [])
        ])
        @obj = mock_model(Obj, :toclist => @children)
        helper.stub!(:cms_path).and_return("/path")
        @current.stub!(:ancestors).and_return [@obj, pub]
      end

      def do_menu
        helper.menu(@obj, @current)
      end

      it "should render a two-level deep menu starting at the given object" do
        do_menu.should have_tag('ul ul', 1)
        do_menu.should have_tag('ul li:last-child ul')
      end

      it "should unfold the branch to the current object" do
        do_menu.should have_tag('li', 3)
        do_menu.should have_tag('li li', 2)
      end

      it "should mark the top-level list with the CSS class 'level-1'" do
        do_menu.should have_tag('ul.level-1')
      end

      it "should mark the second-level list with the CSS class 'level-2'" do
        do_menu.should have_tag('ul ul.level-2')
      end

      it "should mark the first items on each level with CSS class 'first'" do
        do_menu.should have_tag('li:first-child.first>a.first')
        do_menu.should have_tag('li:last-child li:first-child.first>a.first')
      end

      it "should mark the last items on each level with CSS class 'last'" do
        do_menu.should have_tag('li:last-child.last>a.last')
        do_menu.should have_tag('li:last-child li:last-child.last>span.last')
      end

      it "should mark the current item with CSS class 'active'" do
        do_menu.should have_tag('li:last-child li:last-child.active>span.active')
      end

      it "should mark the open items with CSS class 'open'" do
        do_menu.should have_tag('ul.level-1>li:last-child.open>a.open')
        do_menu.should have_tag('ul.level-1>li.open', 1)
      end

      it "should mark the closed items with CSS class 'closed'" do
        do_menu.should have_tag('ul.level-1>li:first-child.closed>a.closed')
      end
    end


    describe "with 5 levels" do

      before do
        @children = []
        @obj = mock_model(Obj, :toclist => @children)
        @children << l1 = mock_model(Obj, :display_title => "L1", :toclist => [
            l2 = mock_model(Obj, :display_title => "L2", :toclist => [
                @current = mock_model(Obj, :display_title => "L3", :toclist => [
                    mock_model(Obj, :display_title => "L4", :toclist => [
                        mock_model(Obj, :display_title => "L5", :toclist => [])
                    ])
                ])
            ])
        ])
        helper.stub!(:cms_path).and_return("/path")
        @current.stub!(:ancestors).and_return [@obj, l1, l2]
      end

      it "should not render deeper than current_obj's children" do
        helper.menu(@obj, @current).should have_tag('ul.level-4')
        helper.menu(@obj, @current).should_not have_tag('ul.level-5')
      end

      it "should render only one level with current_obj == nil" do
        helper.menu(@obj, nil).should have_tag('ul.level-1')
        helper.menu(@obj, nil).should_not have_tag('ul.level-2')
      end

      it "should not render deeper than max_depth" do
        helper.menu(@obj, @current, :max_depth => 2).should have_tag('ul.level-2')
        helper.menu(@obj, @current, :max_depth => 2).should_not have_tag('ul.level-3')
      end

      it "should not render deeper than current_obj's children even if max_depth is higher" do
        helper.menu(@obj, @current, :max_depth => 5).should have_tag('ul.level-4')
        helper.menu(@obj, @current, :max_depth => 5).should_not have_tag('ul.level-5')
      end

      it "should not limit the depth if max_depth is :unlimited" do
        helper.menu(@obj, @current, :max_depth => :unlimited).should have_tag('ul.level-4')
      end

      it "should use the current_time option to evaluate the toclist" do
        current_time = 1.year.from_now
        @obj.should_receive(:toclist).with(current_time).and_return(@children)
        helper.menu(@obj, nil, :current_time => current_time)
      end

      it "should mark the current item with CSS class 'active'" do
        helper.menu(@obj, @current).should have_tag('ul.level-1 ul.level-2 ul.level-3 li.active>span.active')
      end

      it "should mark the open items with CSS class 'open'" do
        helper.menu(@obj, @current).should have_tag('ul.level-1>li.open>a.open')
        helper.menu(@obj, @current).should have_tag('ul.level-2>li.open>a.open')
        helper.menu(@obj, @current).should have_tag('ul.level-3>li.open>span.open')
        helper.menu(@obj, @current).should_not have_tag('ul.level-4>li.open')
      end

      it "should not mark the open items with CSS class 'open'" +
      " if the children are not displayed due to max_depth" do
        helper.menu(@obj, @current, :max_depth => 2).should_not have_tag('ul.level-2>li.open')
      end

      it "should mark the closed items with CSS class 'closed'" do
        helper.menu(@obj, @current).should have_tag('ul.level-4>li.closed>a.closed')
      end

      it "should not mark the closed items with CSS class 'closed'" +
      " if the children are not displayed due to max_depth" do
        helper.menu(@obj, @current, :max_depth => 4).should_not have_tag('ul.level-4>li.closed')
      end

      it "should render the deepest displayable item on current path as link with CSS class 'active'" do
        helper.menu(@obj, @current, :max_depth => 2).should have_tag('ul.level-2>li.active>a.active')
      end

      it "should not give the CSS class 'active' to every item on the current path" do
        helper.menu(@obj, @current).should_not have_tag('ul.level-1>li.active')
        helper.menu(@obj, @current).should_not have_tag('ul.level-2>li.active')
      end
    end


    describe "tree with multiple branches and :unfold => :all" do

      before do
        @children = []
        @obj = mock_model(Obj, :toclist => @children)
        @children << mock_model(Obj, :display_title => "a", :toclist => [
        mock_model(Obj, :display_title => "aa", :toclist => [])])
        @children << mock_model(Obj, :display_title => "b", :toclist => [
        mock_model(Obj, :display_title => "bb", :toclist => [])])
        helper.stub!(:cms_path).and_return("/path")
      end

      def do_menu
        helper.menu(@obj, nil, :unfold => :all)
      end

      it "should render all branches" do
        do_menu.should have_tag("li", 2)
        do_menu.should have_tag("li li", 2)
      end

      it "should mark the open items with CSS class 'open'" do
        do_menu.should have_tag('li.open>a.open', 2)
      end
    end


    describe "called with a block" do

      before do
        @children = []
        @children << mock_model(Obj, :display_title => "child1's title", :sort_value => "Z",
        :toclist => [])
        @children << mock_model(Obj, :display_title => "child2's title", :sort_value => "A",
        :toclist => [])
        @children << mock_model(Obj, :display_title => "child3's title", :sort_value => "B",
        :toclist => [])
        @obj = mock_model(Obj, :toclist => @children)
        helper.stub!(:cms_path).and_return("/path")
      end

      def do_menu
        helper.menu(@obj, nil) do |items, level|
          items.sort_by {|item| item.sort_value}
        end
      end

      it "should renders the items as they are returned by the block" do
        do_menu.should have_tag('ul') do
          with_tag("li:nth-child(1)", "child2's title")
          with_tag("li:nth-child(2)", "child3's title")
          with_tag("li:nth-child(3)", "child1's title")
        end
      end
    end
  end

end
