require File.dirname(__FILE__) + '/../spec_helper'

def create_voting(votes = {})
  votable = Voting.find(:first)
  Vote.delete_all
  votes.each_pair do |answer, count|
    Vote.create(:obj_id => votable.id, :answer => answer, :count => count)
  end
  votable
end

describe Voting, "for an object never voted on before" do

  before do
    @voting = create_voting("maybe" => 7)
    @voting.vote("yes")
  end

  it "should set the vote counter for that particular answer to 1" do
    @voting.count_for_answer("yes").should == 1
  end

  it "should not change the counter for other answers" do
    @voting.count_for_answer("maybe").should == 7
  end

end

describe Voting, "for an object already voted for" do

  before do
    @voting = create_voting("drei" => 1, "zwei" => 9)
    @voting.vote("drei")
  end

  it "should increase the vote counter for that particular answer by 1" do
    @voting.count_for_answer("drei").should == 2
  end

  it "should not change the counter for other answers" do
    @voting.count_for_answer("zwei").should == 9
  end

end

describe Voting, "which has been voted on" do

  before do
    @voting = create_voting("eins" => 10, "vier" => 20)
  end

  it "be flagged as voted" do
    @voting.should be_voted
  end

  it "can be reset" do
    @voting.reset_voting
    @voting.should_not be_voted
  end

  it "should have a total count of answers" do
    @voting.total_votes.should == 30
  end

  it "should return voting results as a percentage per answer" do
    @voting.percent_for_answer("eins").should be_close(33.3, 0.1)
    @voting.percent_for_answer("vier").should be_close(66.6, 0.1)
  end
end


describe Voting, "which has not been voted upon" do

  before do
    @voting = create_voting
    @voting.stub!(:votes).and_return([])
  end

  it "should not be flagged as voted" do
    @voting.should_not be_voted
  end
  it "should have zero votes" do
    @voting.total_votes.should == 0
  end

  it "should return zero voting results per answer" do
    @voting.percent_for_answer("eins").should == 0
    @voting.percent_for_answer("vier").should == 0
  end
end
