# This file is copied to ~/spec when you run 'ruby script/generate rspec'
# from the project root directory.
ENV["RAILS_ENV"] = "test"
require File.expand_path(File.dirname(__FILE__) + "/../config/environment")
require 'spec'
require 'spec/rails'

Spec::Runner.configure do |config|
  # If you're not using ActiveRecord you should remove these
  # lines, delete config/database.yml and disable :active_record
  # in your config/boot.rb
  config.use_transactional_fixtures = true
  config.use_instantiated_fixtures  = false
  config.fixture_path = RAILS_ROOT + '/spec/fixtures/'

  # The problem with testing flash.now is that it gets cleaned up (via #sweep)
  # at the end of the action before you get to test anything.
  # This is a fix from http://rhnh.net/2008/04/19/testing-flash-now-with-rspec
  config.before(:each, :behaviour_type => :controller) do
    @controller.instance_eval { flash.stub!(:sweep) }
  end
end

def mock_obj_model(stubs = {})
  assigns[:obj] = @obj = mock_model(Obj)
  stub_including_bracket_operator(@obj, {
    :body => "Some body text",
    :ancestors => %w( child child child ),
    :has_voting? => false,
    :allow_comments? => false,
    :allow_rating? => true,
    :average_rating_in_percent => 21,
    :display_title => 'display title',
    :ip_description => 'ip_desc',
    :seo_description => 'seo_desc',
    :seo_keywords => 'seo, keywords',
    :permalink => '/cms/path/to/mock/obj',
    :ip_valueAddImage => mock("LinkList", :null_object => true),
    :title => 'title'
  }.merge(stubs))
end

# Stub helpers methods called when rendering a page.
def mock_cms_helpers
  template.stub!(:value_add_image).and_return(nil)
  template.stub!(:value_add_link_list).and_return([])
  template.stub!(:display).and_return("")
end

# Um Specs für eine private Methode zu ermöglichen.
# Achtung: Private Methoden dürfen nur in Ausnahmefällen gespect werden.
# Besser ist es, die privaten Methoden über die public API implizit mitzutesten.
def use_private_method(klass, method)
  raise "#{method} ist keine private Methode von #{klass}" unless klass.private_method_defined?(method)

  before do
    klass.send(:public, method)
  end

  after do
    klass.send(:private, method)
  end
end

# wird benutzt um Obj - Instanzen so zu stubben, dass sie sowohl
# über Punkt- als auch über Klammer-Notation zugegriffen werden können.
def stub_including_bracket_operator(mock, stub_definition)
  stub_definition.each do |message, result|
    mock.stub!(message).and_return(result)
    mock.stub!(:[]).with(message).and_return(result)
  end
end