require File.dirname(__FILE__) + '/../../spec_helper'

describe "/cms/index where the page has not been rated yet by the user" do
  before do
    mock_obj_model
    template.extend RatingsHelper
    template.stub!(:value_add_image).and_return(nil)
    template.stub!(:value_add_link_list).and_return([])
    template.stub!(:display).and_return("")
    template.stub!(:admin?).and_return(false)
    template.stub!(:rated_by_current_user?).and_return(false)
    render "cms/index", :layout => 'cms'
  end

  it "should display a ratings widget" do
    response.should have_tag('div#starRating')
  end

  it "should show how the other people have rated" do
    response.should have_tag('li.current-rating[style*=width:21%]')
  end

  it "should show 5 clickable stars" do
    response.should have_tag('li a.one-star')
  end

  it "should invite the user to rate" do
    response.should have_tag('div#starRating_label', 'Bewerten Sie hier!')
  end
end


describe "/cms/index where the page has already been rated by the user" do
  before do
    mock_obj_model
    template.extend RatingsHelper
    template.stub!(:value_add_image).and_return(nil)
    template.stub!(:value_add_link_list).and_return([])
    template.stub!(:display).and_return("")
    template.stub!(:admin?).and_return(false)
    template.stub!(:rated_by_current_user?).and_return(true)
    render "cms/index", :layout => 'cms'
  end

  it "should display a ratings widget" do
    response.should have_tag('div#starRating')
  end

  it "should show how the other people have rated" do
    response.should have_tag('li.current-rating[style~=width:21%;]')
  end

  it "should not show the 5 clickable stars" do
    response.should_not have_tag('li a.one-star')
  end

  it "should thank the user for rating" do
    response.should have_tag('div#starRating_label', 'Danke für Ihre Bewertung.')
  end
end


describe "/cms/index where the page is not meant to be rated" do
  before do
    mock_cms_helpers
    mock_obj_model :allow_rating? => false
    render "cms/index", :layout => 'cms'
  end

  it "should not display a ratings widget" do
    response.should_not have_tag('div#starRating')
  end
end


describe "/cms/index where the user is admin" do
  before do
    mock_obj_model
    template.extend RatingsHelper
    template.stub!(:value_add_image).and_return(nil)
    template.stub!(:value_add_link_list).and_return([])
    template.stub!(:display).and_return("")
    template.stub!(:admin?).and_return(true)
    template.stub!(:rated_by_current_user?).and_return(false)
    render "cms/index", :layout => 'cms'
  end

  it "should be able to reset the ratings" do
    response.should have_tag('.ratings_admin')
    response.should have_tag('.ratings_admin a')
  end
end


describe "/cms/index where the user is an ordinary user" do
  before do
    mock_obj_model
    template.extend RatingsHelper
    template.stub!(:value_add_image).and_return(nil)
    template.stub!(:value_add_link_list).and_return([])
    template.stub!(:display).and_return("")
    template.stub!(:admin?).and_return(false)
    template.stub!(:rated_by_current_user?).and_return(false)
    render "cms/index"
  end

  it "should not be able to reset the ratings" do
    response.should_not have_tag('.ratings_admin')
    response.should_not have_tag('.ratings_admin a')
  end
end
