require File.dirname(__FILE__) + '/../../spec_helper'

describe "All show pages", :shared => true do
  before do
    assigns[:obj] = @obj = mock_model(Wiki, :to_param => 123)
    assigns[:page] = @page = mock_model(WikiPage, :body => "Some content for the other page", :name => "Page")
    template.stub!(:format_wiki_text).and_return("foo")
  end

  def do_render
    render "/wiki/show"
  end

  it "should have a link to edit the page" do
    do_render
    response.should have_tag("a[href='/wiki/123/edit/Page']")
  end

  it "should have a link to list all pages" do
    do_render
    response.should have_tag("a[href='/wiki/123/list']")
  end

  it "should have a link to the start page" do
    do_render
    response.should have_tag("a[href='/wiki/123']")
  end
end


describe "show existing page" do
  it_should_behave_like "All show pages"

  before do
    @page.stub!(:new_record?).and_return(false)
  end

  it "should display the formatted Wiki page content" do
    template.should_receive(:format_wiki_text).with(@page.body).and_return("content")
    do_render
    response.should have_tag('div', "content")
  end

  it "should not render a blank slate" do
    do_render
    response.should_not have_tag("div[class='wiki_blank_slate']")
  end

  it "should have a delete link" do
    do_render
    response.should have_tag("a[onclick*='http://test.host/wiki/123/delete/Page']")
  end
end


describe "show new page" do
  it_should_behave_like "All show pages"

  before do
    @page.stub!(:new_record?).and_return(true)
  end

  it "should render a blank slate" do
    do_render
    response.should have_tag("div[class='wiki_blank_slate']")
  end

  it "should not have a delete link" do
    do_render
    response.should_not have_tag("a[onclick*='http://test.host/wiki/123/delete/Page']")
  end
end
